@@ -1,58 +1,111 @@
 package DateTime::Format::Alami;
 
-our $DATE = '2016-05-05'; # DATE
-our $VERSION = '0.09'; # VERSION
+our $DATE = '2016-06-28'; # DATE
+our $VERSION = '0.10'; # VERSION
 
 use 5.010001;
 use strict;
 use warnings;
-use experimental 'smartmatch';
+use Log::Any::IfLOG '$log';
 
-my @shortmons = qw(jan feb mar apr may jun jul aug sep oct nov dec);
+use Role::Tiny;
 
-# must be overriden
-sub o_num       {}
-sub _parse_num  {}
-sub w_year   {}
-sub w_month  {}
-sub w_week   {}
-sub w_day    {}
-sub w_minute {}
-sub w_second {}
-sub p_now       {}
-sub p_today     {}
-sub p_tomorrow  {}
-sub p_yesterday {}
-sub p_dur_ago   {}
-sub p_dur_later {}
+my @short_mons = qw(jan feb mar apr may jun jul aug sep oct nov dec);
 
+requires 'o_num';
+requires '_parse_num';
+
+requires 'w_year';
+requires 'w_month';
+requires 'w_week';
+requires 'w_day';
+requires 'w_hour';
+requires 'w_minute';
+requires 'w_second';
+
+requires "w_$_" for @short_mons;
+
+requires 'p_now';
+requires 'p_today';
+requires 'p_yesterday';
+requires 'p_tomorrow';
+requires 'p_dateymd';
+requires 'o_date';
+requires 'p_dur_ago';
+requires 'p_dur_later';
+requires 'p_time';
+requires 'p_date_time';
+
+our ($m, $o);
 sub new {
     my $class = shift;
     if ($class eq __PACKAGE__) {
-        die "Use one of my subclasses instead, ".
+        die "Use one of the DateTime::Format::Alami::* instead, ".
             "e.g. DateTime::Format::Alami::EN";
     }
     my $self = bless {}, $class;
     no strict 'refs';
     unless (${"$class\::RE"}) {
         require Class::Inspector;
+        require Data::Graph::Util;
+
         my $meths = Class::Inspector->methods($class);
-        my @pats;
-        for (@$meths) {
-            next unless /^p_/;
-            my $pat = $self->$_;
-            $pat =~ s/<(\w+)>/"(?P<$1>" . $self->$1 . ")"/eg;
-            push @pats, "(?P<$_>$pat)";
+        my %pats;  # key = "p_..."
+        my %pat_lengths; # key = "p_..."
+        my %graph;
+        for my $meth (@$meths) {
+            next unless $meth =~ /^[op]_/;
+            my $pat = $self->$meth;
+            my $is_p = $meth =~ /^p_/;
+            $pat =~ s/<(\w+)>/push @{$graph{$meth}}, $1; "(?\&$1)"/eg;
+            my $action_meth = $meth; $action_meth =~ s/^p_/a_/;
+            my $before_meth = $meth; $before_meth =~ s/^p_/before_p_/;
+            $before_meth = undef unless $is_p && $self->can($before_meth);
+            $pat = join(
+                "",
+                "(",
+                ($before_meth ? "(?{ ".($ENV{DEBUG} ? "say \"invoking $before_meth()\";" : "")."\$DateTime::Format::Alami::o->$before_meth(\$DateTime::Format::Alami::m) })" : ""),
+                ($is_p ? "\\b $pat \\b" : $pat), ")",
+
+                # we capture ourselves instead of relying on named capture
+                # because subpattern capture are discarded
+                "(?{ \$DateTime::Format::Alami::m->{$meth} = \$^N })",
+
+                ($is_p ? "(?{ ".($ENV{DEBUG} ? "say \"invoking $action_meth(\$^N)\";" : "")."\$DateTime::Format::Alami::o->{_pat} = \"$meth\"; \$DateTime::Format::Alami::o->$action_meth(\$DateTime::Format::Alami::m) })" : ""),
+            );
+            $pats{$meth}  = $pat;
+            $pat_lengths{$meth} = length($pat);
+        }
+        my @pat_names_by_deps = Data::Graph::Util::toposort(\%graph);
+        my %pat_name_dep_orders = map { $pat_names_by_deps[$_] => $_ }
+            0..$#pat_names_by_deps;
+        my @pat_names = sort {(
+            ($pat_name_dep_orders{$a} // 9999) <=>
+                ($pat_name_dep_orders{$b} // 9999)
+                ||
+                $pat_lengths{$b} <=> $pat_lengths{$a}) } keys %pats;
+        my $nl = $ENV{DEBUG} ? "\n" : "";
+        my $re = join(
+            "",
+            "(?&top)", $nl,
+            #"(?&p_dateymd)", $nl, # testing
+            "(?(DEFINE)", $nl,
+            "(?<top>", join("|",
+                            map {"(?&$_)"} grep {/^p_/} @pat_names), ")$nl",
+            (map { "(?<$_> $pats{$_})$nl" } @pat_names),
+            ")", # end of define
+        );
+        {
+            use re 'eval';
+            ${"$class\::RE"} = qr/$re/ix;
         }
-        my $re = join("|", sort {length($b)<=>length($a)} @pats);
-        ${"$class\::RE"} = qr/\b($re)\b/ix;
     }
     unless (${"$class\::MAPS"}) {
         my $maps = {};
         # month names -> num
         {
             my $i = 0;
-            for my $m (@shortmons) {
+            for my $m (@short_mons) {
                 ++$i;
                 my $meth = "w_$m";
                 for (@{ $self->$meth }) {
@@ -65,7 +118,15 @@ sub new {
     $self;
 }
 
+sub _reset {
+    my $self = shift;
+    undef $self->{_pat};
+    undef $self->{_dt};
+    undef $self->{_uses_time};
+}
+
 sub parse_datetime {
+    require DateTime;
     no strict 'refs';
 
     my ($self, $str, $opts) = @_;
@@ -79,14 +140,24 @@ sub parse_datetime {
 
     my $re = ${ref($self).'::RE'};
 
+    $o = $self;
     my @res;
-    while ($str =~ /$re/go) {
-        my %m = %+;
-        push @res, {
+    while (1) {
+        $o->_reset;
+        $m = {};
+        $str =~ /($re)/go or last;
+        $o->{_dt}->truncate(to=>'day') unless $o->{_uses_time};
+        my $res = {
             verbatim => $1,
+            pattern => $o->{_pat},
             pos => pos($str) - length($1),
-            m => \%m,
+            m => {%$m},
         };
+        $res->{uses_time} = $o->{_uses_time} ? 1:0;
+        $res->{DateTime}  = $o->{_dt};
+        $res->{epoch}     = $o->{_dt}->epoch if
+            $opts->{format} eq 'combined' || $opts->{format} eq 'epoch';
+        push @res, $res;
         last if $opts->{returns} eq 'first';
     }
 
@@ -94,35 +165,6 @@ sub parse_datetime {
 
     @res = ($res[-1]) if $opts->{returns} eq 'last';
 
-    my $can_skip_calculate_dt = $opts->{format} eq 'verbatim' &&
-        $opts->{returns} =~ /\A(?:first|last|all)\z/;
-
-    for my $res (@res) {
-        for (keys %{$res->{m}}) {
-            if (/^p_(.+)/) {
-                $res->{pattern} = $1;
-                unless ($can_skip_calculate_dt) {
-                    my $meth = "a_$1";
-
-                    # reset state for a_* method
-                    undef $self->{_dt};
-                    undef $self->{_uses_time};
-
-                    $self->$meth($res->{m});
-                }
-                last;
-            }
-        }
-
-        unless ($can_skip_calculate_dt) {
-            $self->{_dt}->truncate(to => 'day') unless $self->{_uses_time};
-            $res->{DateTime} = $self->{_dt};
-            $res->{uses_time} = $self->{_uses_time} ? 1:0;
-            $res->{epoch} = $self->{_dt}->epoch if
-                $opts->{format} eq 'combined' || $opts->{format} eq 'epoch';
-        }
-    }
-
     if ($opts->{returns} =~ /\A(?:all_cron|earliest|latest)\z/) {
         # sort chronologically, note that by this time the DateTime module
         # should already have been loaded
@@ -156,11 +198,17 @@ sub o_monthint { "(?:0?[1-9]|1[012])" }
 
 sub o_yearint { "(?:[0-9]{4}|[0-9]{2})" }
 
+sub o_hour { "(?:[0-9][0-9]?)" }
+
+sub o_minute { "(?:[0-9][0-9]?)" }
+
+sub o_second { "(?:[0-9][0-9]?)" }
+
 sub o_monthname {
     my $self = shift;
     "(?:" . join(
         "|",
-        (map {my $meth="w_$_"; @{ $self->$meth }} @shortmons)
+        (map {my $meth="w_$_"; @{ $self->$meth }} @short_mons)
     ) . ")";
 }
 
@@ -173,6 +221,7 @@ sub o_durwords  {
         @{ $self->w_hour }, @{ $self->w_minute }, @{ $self->w_second },
     ) . ")";
 }
+
 sub o_dur {
     my $self = shift;
     "(?:(" . $self->o_num . "\\s*" . $self->o_durwords . "\\s*)+)";
@@ -186,12 +235,15 @@ sub o_timedurwords  {
         @{ $self->w_hour }, @{ $self->w_minute }, @{ $self->w_second },
     ) . ")";
 }
+
 sub o_timedur {
     my $self = shift;
     "(?:(" . $self->o_num . "\\s*?" . $self->o_timedurwords . "\\s*)+)";
 }
 
 sub _parse_dur {
+    use experimental 'smartmatch';
+
     my ($self, $str) = @_;
 
     #say "D:dur=$str";
@@ -232,60 +284,51 @@ sub _parse_dur {
             $args{years} = $n;
         }
     }
+    require DateTime::Duration;
     DateTime::Duration->new(%args);
 }
 
-sub _setif_now {
+sub _now_if_unset {
     my $self = shift;
-    unless ($self->{_dt}) {
-        require DateTime;
-        $self->{_dt} = DateTime->now(
-            (time_zone => $self->{_time_zone}) x !!defined($self->{_time_zone}),
-        );
-    }
+    $self->a_now unless $self->{_dt};
 }
 
-sub _setif_today {
+sub _today_if_unset {
     my $self = shift;
-    unless ($self->{_dt}) {
-        require DateTime;
-        $self->{_dt} = DateTime->today(
-            (time_zone => $self->{_time_zone}) x !!defined($self->{_time_zone}),
-        );
-    }
+    $self->a_today unless $self->{_dt};
 }
 
 sub a_now {
     my $self = shift;
-    $self->_setif_now;
+    $self->{_dt} = DateTime->now(
+        (time_zone => $self->{_time_zone}) x !!defined($self->{_time_zone}),
+    );
     $self->{_uses_time} = 1;
 }
 
 sub a_today {
     my $self = shift;
-    $self->_setif_today;
-}
-
-sub a_timedur_today {
-    my $self = shift;
-    $self->_setif_today;
+    $self->{_dt} = DateTime->today(
+        (time_zone => $self->{_time_zone}) x !!defined($self->{_time_zone}),
+    );
+    $self->{_uses_time} = 0;
 }
 
 sub a_yesterday {
     my $self = shift;
-    $self->_setif_today;
+    $self->a_today;
     $self->{_dt}->subtract(days => 1);
 }
 
 sub a_tomorrow {
     my $self = shift;
-    $self->_setif_today;
+    $self->a_today;
     $self->{_dt}->add(days => 1);
 }
 
-sub a_date_ymd {
+sub a_dateymd {
     my ($self, $m) = @_;
-    $self->_setif_now;
+    $self->a_today;
     if (defined $m->{o_yearint}) {
         my $year;
         if (length($m->{o_yearint}) == 2) {
@@ -311,18 +354,36 @@ sub a_date_ymd {
 
 sub a_dur_ago {
     my ($self, $m) = @_;
-    $self->_setif_now;
+    $self->a_now;
     my $dur = $self->_parse_dur($m->{o_dur});
     $self->{_dt}->subtract_duration($dur);
 }
 
 sub a_dur_later {
     my ($self, $m) = @_;
-    $self->_setif_now;
+    $self->a_now;
     my $dur = $self->_parse_dur($m->{o_dur});
     $self->{_dt}->add_duration($dur);
 }
 
+sub a_time {
+    my ($self, $m) = @_;
+    $self->_now_if_unset;
+    $self->{_uses_time} = 1;
+    my $hour = $m->{o_hour};
+    if ($m->{o_ampm}) {
+        $hour += 12 if lc($m->{o_ampm}) eq 'pm' && $hour < 12;
+        $hour =  0  if lc($m->{o_ampm}) eq 'am' && $hour == 12;
+    }
+    $self->{_dt}->set_hour($hour);
+    $self->{_dt}->set_minute($m->{o_minute});
+    $self->{_dt}->set_second($m->{o_second} // 0);
+}
+
+sub a_date_time {
+    my ($self, $m) = @_;
+}
+
 1;
 # ABSTRACT: Parse human date/time expression (base class)
 
@@ -338,7 +399,7 @@ DateTime::Format::Alami - Parse human date/time expression (base class)
 
 =head1 VERSION
 
-This document describes version 0.09 of DateTime::Format::Alami (from Perl distribution DateTime-Format-Alami), released on 2016-05-05.
+This document describes version 0.10 of DateTime::Format::Alami (from Perl distribution DateTime-Format-Alami), released on 2016-06-28.
 
 =head1 SYNOPSIS
 
@@ -360,17 +421,10 @@ use Indonesian:
 
 =head1 DESCRIPTION
 
-B<EARLY RELEASE: PROOF OF CONCEPT ONLY AND VERY VERY INCOMPLETE.>
-
 This class parses human/natural date/time string and returns DateTime object.
 Currently it supports English and Indonesian. The goal of this module is to make
 it easier to add support for other human languages.
 
-It works by matching date string with a bunch of regex patterns (assembled from
-C<p_*> methods, e.g. C<p_today>, C<p_dur_ago>, C<p_dur_later>, and so on). If a
-pattern is found, the corresponding C<a_*> method is called to compute the
-DateTime object (e.g. if C<p_today> pattern matches, C<a_today> is called).
-
 To actually use this class, you must use one of its subclasses for each
 human language that you want to parse.
 
@@ -379,6 +433,60 @@ elsewhere, see L</"SEE ALSO">.
 
 =for Pod::Coverage ^([aopw]_.+)$
 
+=head1 HOW IT WORKS
+
+L<DateTime::Format::Alami> is base class. Each human language is implemented in
+a separate C<< DateTime::Format::Alami::<ISO_CODE> >> module (e.g.
+L<DateTime::Format::Alami::EN> and L<DateTime::Format::Alami::EN>) which is a
+subclass.
+
+Parsing is done using a single recursive regex (i.e. containing C<(?&NAME)> and
+C<(?(DEFINE))> patterns, see L<perlre>). This regex is composed from pieces of
+pattern strings in the C<p_*> and C<o_*> methods, to make it easier to override
+in an OO-fashion.
+
+A pattern string that is returned by the C<p_*> method is a normal regex pattern
+string that will be compiled using the /x and /i regex modifier. The pattern
+string can also refer to pattern in other C<o_*> or C<p_*> method using syntax
+C<< <o_foo> >> or C<< <p_foo> >>. Example, C<o_today> for English might be
+something like:
+
+ sub p_today { "(?: today | this \s+ day )" }
+
+Other examples:
+
+ sub p_yesterday { "(?: yesterday )" }
+
+ sub p_dateymd { join(
+     "",
+    '(?: <o_dayint> \\s* ?<o_monthname> | <o_monthname> \\s* <o_dayint>\\b|<o_monthint>[ /-]<o_dayint>\\b )',
+    '(?: \\s*[,/-]?\\s* <o_yearint>)?'
+ )}
+
+ sub o_date { "(?: <p_today>|<p_yesterday>|<p_dateymd>)" }
+
+ sub p_time { "(?: <o_hour>:<o_minute>(?:<o_second>)? \s* <o_ampm> )" }
+
+ sub p_date_time { "(?: <o_date> (?:\s+ at)? <o_time> )" }
+
+When a pattern from C<p_*> matches, a corresponding action method C<a_*> will be
+invoked. Usually the method will set or modify a DateTime object in C<<
+$self->{_dt} >>. For example, this is code for C<a_today>:
+
+ sub a_today {
+     my $self = shift;
+     $self->{_dt} = DateTime->today;
+ }
+
+The patterns from all C<p_*> methods will be combined in an alternation to form
+the final pattern.
+
+An C<o_*> pattern is just like C<p_*>, but they will not be
+combined into the final pattern and matching it won't execute a corresponding
+C<a_*> method.
+
+And there are also C<w_*> methods which return array of strings.
+
 =head1 ADDING A NEW HUMAN LANGUAGE
 
 TBD
